// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SimpleTexturedShader.hpp"

using namespace Dawn;

void SimpleTexturedShader::compile() {
  this->program.compile();
}

std::vector<struct ShaderPass> SimpleTexturedShader::getItemPasses(
  MeshRenderer *mesh, Material *material
) {
  SimpleTexturedMaterial *simpleMaterial = dynamic_cast<SimpleTexturedMaterial*>(material);
  assertNotNull(simpleMaterial);

  struct ShaderPass onlyPass;
  onlyPass.shaderProgram = &program;
  onlyPass.colorValues[program.paramColor] = simpleMaterial->color;
  onlyPass.matrixValues[program.paramModel] = mesh->transform->getWorldTransform();

  if(simpleMaterial->texture != nullptr) {
    onlyPass.boolValues[program.paramHasTexture] = true;
    onlyPass.textureSlots[0] = simpleMaterial->texture;
    onlyPass.textureValues[program.paramTexture] = 0;
  } else {
    onlyPass.boolValues[program.paramHasTexture] = false;
  }

  std::vector<struct ShaderPass> passes;
  passes.push_back(onlyPass);
  return passes;
}

void SimpleTexturedShader::setGlobalParameters(
  glm::mat4 cameraProjection, glm::mat4 cameraView
) {
  this->program.setMatrix(this->program.paramProjection, cameraProjection);
  this->program.setMatrix(this->program.paramView, cameraView);
}